COMMON SHARE1,nx,ny,nz
COMMON SHARE2,x,y,z,vx,vy,vz,d,p,e
;------------------------------------------------------------------------------
; Function TEST_ISHDF: Tests that file is valid HDF4 SDS
;
FUNCTION TEST_ISHDF, filename
CATCH, err
IF (err EQ 0) THEN RETURN, HDF_ISHDF(filename) $
          ELSE RETURN, 0
END
;------------------------------------------------------------------------------
; Procedure READHDF: Reads HDF4 SDS files generated by cmhog3.0
;  These files always contain vx,vy,vz,d,E,p in that order.
;  The x,y, and z-coordinates are also stored with each data block, they
;   are read from the first block.  The coordinates may have an arbitrary
;   number of elements, corresponding to nx, ny, and nz.
;
pro readhdf, filename
COMMON SHARE1,nx,ny,nz
COMMON SHARE2,x,y,z,vx,vy,vz,d,p,e
;
; Open file and initialize the SDS interface
;
IF NOT TEST_ISHDF(filename) THEN BEGIN
  PRINT, 'Invalid HDF file...'
  RETURN
  ENDIF ELSE $
  PRINT, 'Valid HDF file. Opening "' + filename + '"'
newfileID=HDF_SD_START(filename, /READ)
HDF_SD_FILEINFO, newfileID, ndatasets, attributes
;
; Read z (first data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 0)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index0=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID0=HDF_SD_SELECT(newfileID, index0)
HDF_SD_GETDATA, sdsID0, z
nz = SIZE(z,/N_ELEMENTS)
;
; Read y (second data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 1)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index1=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID1=HDF_SD_SELECT(newfileID, index1)
HDF_SD_GETDATA, sdsID1, y
ny = SIZE(y,/N_ELEMENTS)
;
; Read x (third data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 2)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index2=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID2=HDF_SD_SELECT(newfileID, index2)
HDF_SD_GETDATA, sdsID2, x
nx = SIZE(x,/N_ELEMENTS)
print,'nx=',nx,' ny=',ny,' nz=',nz
;
; Read vx (fourth data set)
;
thisSDS=HDF_SD_SELECT(newfileID, 3)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index3=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID3=HDF_SD_SELECT(newfileID, index3)
HDF_SD_GETDATA, sdsID3, vx
;
; Now read every fourth data set to get vy,vz,d,p,E (skipping the x,y,z
; stored with each variable)
;
thisSDS=HDF_SD_SELECT(newfileID, 7)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index7=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID7=HDF_SD_SELECT(newfileID, index7)
HDF_SD_GETDATA, sdsID7, vy
;
thisSDS=HDF_SD_SELECT(newfileID, 11)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index11=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID11=HDF_SD_SELECT(newfileID, index11)
HDF_SD_GETDATA, sdsID11, vz
;
thisSDS=HDF_SD_SELECT(newfileID, 15)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index15=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID15=HDF_SD_SELECT(newfileID, index15)
HDF_SD_GETDATA, sdsID15, d
;
thisSDS=HDF_SD_SELECT(newfileID, 19)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index19=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID19=HDF_SD_SELECT(newfileID, index19)
HDF_SD_GETDATA, sdsID19, e
;
thisSDS=HDF_SD_SELECT(newfileID, 23)
HDF_SD_GETINFO, thisSDS, NAME=thisSDSName, NATTS=numAttributes
index23=HDF_SD_NAMETOINDEX(newfileID, thisSDSName)
sdsID23=HDF_SD_SELECT(newfileID, index23)
HDF_SD_GETDATA, sdsID23, p
;
;
;  Close HDF file and end
;
HDF_SD_END, newfileID
END
;------------------------------------------------------------------------------
; Procedure PLOT1 : Makes 1D plot of single variable using points
;
PRO plot1,coord,var

IF (SIZE(var,/N_DIMENSIONS) NE 1) THEN BEGIN
  PRINT, 'Variable is not a 1-D array...'
  RETURN
ENDIF
IF (SIZE(coord,/N_ELEMENTS) NE SIZE(var,/N_ELEMENTS)) THEN BEGIN
  PRINT, 'Coordinate and variable have different number of elements...'
  RETURN
ENDIF

ymin=MIN(var)
ymax=MAX(var)
plot,coord,var,psym=6,symsize=.4,YRANGE=[ymin,ymax]

END
;------------------------------------------------------------------------------
; Procedure PLOT2 : Makes 2D contour plot of single variable with 30 levels
;
PRO plot2,coord1,coord2,var

sizes = SIZE(var)
IF (sizes[0] NE 2) THEN BEGIN
  PRINT, 'Variable is not a 2-D array...'
  RETURN
ENDIF
IF (SIZE(coord1,/N_ELEMENTS) NE sizes[1]) THEN BEGIN
  PRINT, 'Coordinate1 and leading dimension of variable have different number of elements...'
  RETURN
ENDIF
IF (SIZE(coord2,/N_ELEMENTS) NE sizes[2]) THEN BEGIN
  PRINT, 'Coordinate2 and second dimension of variable have different number of elements...'
  RETURN
ENDIF

contour,var,coord1,coord2,NLEVELS=30

END



